// ---------------------------------------------------
//                 Class ThreadDemo
// ---------------------------------------------------
// Made by your Dearest Teacher.
//
// A demo using threads with a bunch of balloons that
// jump around in a window. You can control the speed
// of each of the balloons and the refresh rate of
// their panel.
// ---------------------------------------------------

// ---------------------------------------------------
// Standard GUI imports
// ---------------------------------------------------
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

// ---------------------------------------------------
// For the JSlider and events
// ---------------------------------------------------
import javax.swing.event.*;

// ---------------------------------------------------
// To display the balloons
// ---------------------------------------------------
import java.awt.geom.*;
import java.util.*;

// ---------------------------------------------------
// ---------------------------------------------------
// Our program
// ---------------------------------------------------
// ---------------------------------------------------
public class ThreadDemo extends JFrame
{
   private static final String
      DEFAULT_TITLE  = "Thread Demo"
,     STATUS_DEFAULT = "Click (and hold) anywhere in the window to create a balloon"
,     STATUS_FULL    = "Okay, that's enough!"
;
   // ------------------------------------------------
   // I have set the maximum number of balloons to
   // this value, but it's an arbitrary decision.
   // You can change it if you want.
   // ------------------------------------------------
   private static final int
      MAX_BALLOONS = 5; // arbitrary

   // ------------------------------------------------
   // For some randomness...
   // ------------------------------------------------
   private static Random r = new Random ();

   // ------------------------------------------------
   // The current number of balloons ...
   // ------------------------------------------------
   private int NbBalloons;

   private void SetNbBalloons (int n)
   {
      NbBalloons = n;
   }
   public int GetNbBalloons ()
   {
      return NbBalloons;
   }
   public boolean IsFullOfBalloons ()
   {
      return GetNbBalloons () == MAX_BALLOONS;
   }


   // ------------------------------------------------
   // StatusLabel displays the messages at the
   // bottom of the window, using SetStatus(String)
   // ------------------------------------------------
   private JLabel StatusLabel;

   public void SetStatus (String statusText)
   {
      StatusLabel.setText (statusText + " (" + GetNbBalloons () +
                           " so far, max: " + MAX_BALLOONS + ")");
   }

   // ------------------------------------------------
   // ------------------------------------------------
   // BalloonWorld displays the balloons. It is of
   // class BalloonPanel, a subclass of JPanel that I
   // have made just for this and in which I have
   // overriden the paint(Graphics) method to make
   // sure it displays the balloons properly.
   // ------------------------------------------------
   // ------------------------------------------------
   private BalloonPanel BalloonWorld;

   private class BalloonPanel extends JPanel
   {
      // ---------------------------------------------
      // The background color of a BalloonPanel
      // ---------------------------------------------
      private final Color
         BACKGROUND_COLOR = Color.black;

      // ---------------------------------------------
      // Whenever a new balloon is added to the panel,
      // it is given a color from the array below. If
      // there are more balloons in there than there
      // are entries in this array, I go back to the
      // first color (it doesn't crash).
      // ---------------------------------------------
      private final Color [] BALLOON_COLOR =
      {
         Color.white, Color.red,  Color.blue, Color.green, Color.yellow,
         Color.pink,  Color.cyan, Color.magenta
      };

      // ---------------------------------------------
      // Default constructor
      // ---------------------------------------------
      public BalloonPanel ()
      {
         super ();
         setBackground (BACKGROUND_COLOR);
      }

      // ---------------------------------------------
      // The paint(Graphics) method. This is where the
      // balloons are effectively displayed.
      // ---------------------------------------------
      public void paint (Graphics g)
      {
         // ----------------------------------------------
         // To make sure the standard display code is
         // well executed...
         // ----------------------------------------------
         super.paint (g);

         // ----------------------------------------------
         // Within this method, I will be using the 2D
         // drawing utilities. Check the book for details
         // ----------------------------------------------
         Graphics2D g2D = (Graphics2D) g;

         // ----------------------------------------------
         // For each balloon (there might be none).
         // ----------------------------------------------
         for (int i = 0; i < GetNbBalloons (); i++)
         {
            // ----------------------------------------------
            // Checking collisions with the borders of the
            // panel (between 0 and getWidth()). Note that
            // we must use the center +/- half of the radius
            // of a balloon to check correctly for contact
            // with a balloon's edge...
            // ----------------------------------------------

            int HalfRadius = Balloons[i].GetRadius () / 2;

            // Collisions along the X axis
            if (Balloons[i].GetCenterX () + HalfRadius >= getWidth  ())
            {
               Balloons[i].SetCenterX (getWidth () - HalfRadius - 1);
               Balloons[i].SetIsCollidingX (true);
            }
            else if (Balloons[i].GetCenterX () - HalfRadius <= 0)
            {
               Balloons[i].SetCenterX (HalfRadius + 1);
               Balloons[i].SetIsCollidingX (true);
            }
            else
            {
               Balloons[i].SetIsCollidingX (false);
            }

            // Collisions along the Y axis
            if (Balloons[i].GetCenterY () + HalfRadius >= getHeight ())
            {
               Balloons[i].SetCenterY (getHeight () - HalfRadius - 1);
               Balloons[i].SetIsCollidingY (true);
            }
            else if (Balloons[i].GetCenterY () - HalfRadius <= 0)
            {
               Balloons[i].SetCenterY (HalfRadius + 1);
               Balloons[i].SetIsCollidingY (true);
            }
            else
            {
               Balloons[i].SetIsCollidingY (false);
            }

            // ----------------------------------------------
            // Drawing the balloon (a circle, which means a
            // special kind of ellipse).
            // ----------------------------------------------
            g2D.setPaint (BALLOON_COLOR[i % BALLOON_COLOR.length]);
            g2D.fill (new Ellipse2D.Float ((float) (Balloons[i].GetCenterX () - HalfRadius),
                                           (float) (Balloons[i].GetCenterY () - HalfRadius),
                                           (float) Balloons[i].GetRadius (),
                                           (float) Balloons[i].GetRadius ()));
         }
      }
   }
   // ------------------------------------------------
   // ------------------------------------------------




   // ------------------------------------------------
   // ------------------------------------------------
   // Class SliderPanel. I use sliders to control some
   // things like balloon speed and refresh rate. I
   // have decided to create a subclass of JPanel that
   // I have named SliderPanel.
   //
   // This class represents a panel with a slider, a
   // title, and three labels to indicate the min, max
   // and current values of the slider.
   //
   // The panel registers itself as a listener for its
   // own slider, and updates its own display when the
   // slider is manipulated.
   //
   // Creating this class has very much simplfied my
   // program...
   // ------------------------------------------------
   // ------------------------------------------------

   // ------------------------------------------------
   // RefreshSlider will be the display's refresh rate
   // controller. With it, the user will have control
   // over how many times per second the BalloonPanel
   // is to be refreshed.
   //
   // The Sliders[] array will contain one SliderPanel
   // per balloon. Each will let the user control its
   // balloon's speed.
   // ------------------------------------------------
   private SliderPanel RefreshSlider;
   private SliderPanel [] Sliders;

   private class SliderPanel extends JPanel implements ChangeListener
   {
      // ---------------------------------------------
      // sl will be the slider on the panel
      // ---------------------------------------------
      private JSlider sl;

      // ---------------------------------------------
      // These labels will display pertinent data on
      // the state of the slider. The Cur label is
      // to be updated every time the slider's value
      // will change.
      // ---------------------------------------------
      private JLabel Title, Min, Max, Cur;

      // ---------------------------------------------
      // Constructor. Creates the whole panel, places
      // the controls on it, and registers the instance
      // under construction as change listener on the
      // panel's slider.
      // ---------------------------------------------
      public SliderPanel (String title, int min, int max, int init)
      {
         super ();

         final int MIDDLE_NB_ROWS    = 1,
                   MIDDLE_NB_COLUMNS = 3,
                   PANEL_NB_ROWS     = 3,
                   PANEL_NB_COLUMNS  = 1;

         Title = new JLabel (title, JLabel.CENTER);
         sl = new JSlider ( min, max, init);
         Min = new JLabel ("Min: " + min, JLabel.LEFT);
         Max = new JLabel ("Max: " + max, JLabel.RIGHT);
         Cur = new JLabel ("Cur: " + init, JLabel.CENTER);

         setLayout (new GridLayout (PANEL_NB_ROWS, PANEL_NB_COLUMNS));

         // At the top of the panel: a title
         add (Title);

         // On the middle of the panel: info on the slider
         JPanel Middle = new JPanel ();
         Middle.setLayout (new GridLayout (MIDDLE_NB_ROWS, MIDDLE_NB_COLUMNS));
         Middle.add (Min);
         Middle.add (Cur);
         Middle.add (Max);
         add (Middle);

         // At the bottom of the panel: the slider
         add (sl);
         sl.addChangeListener (this);
      }

      // ---------------------------------------------
      // To facilitate the life of the other classes (I should
      // have encapsulated the slider more)...
      // ---------------------------------------------
      private JSlider GetSlider ()
      {
         return sl;
      }

      // ---------------------------------------------
      // Event handling on slider value change...
      // ---------------------------------------------
      public void stateChanged (ChangeEvent e)
      {
         Cur.setText ("Cur: " + GetSlider ().getValue ());
      }
   }
   // ------------------------------------------------
   // ------------------------------------------------


   // ------------------------------------------------
   // ------------------------------------------------
   // Class PainterThread performs a periodic update
   // of the display of BalloonThread. Its rythm can
   // be controlled by the user (with RefreshSlider).
   //
   // ThePainter is the instance of PainterThread used
   // by our ThreadDemo instance
   // ------------------------------------------------
   // ------------------------------------------------
   private PainterThread ThePainter;

   private class PainterThread extends Thread implements ChangeListener
   {
      private static final long
         DEFAULT_REPAINT_INTERVAL = 50  // ten times per second
;
      // ---------------------------------------------
      // The RepaintInterval is in milliseconds.
      // Attribute Stopping will be true when the thread
      // will try to stop itself, and will be reset to
      // false once stopping has performed
      // ---------------------------------------------
      private long RepaintInterval;     // in milliseconds
      private boolean Stopping;

      // ---------------------------------------------
      // Accessors (Set)
      // ---------------------------------------------

      private void SetStopping (boolean shouldStop)
      {
         Stopping = shouldStop;
      }

      // ---------------------------------------------
      // One can set the repaint interval in terms of
      // in how many milliseconds will it wake up?,
      // or in terms of how many times per second will
      // it wake up? (both are different ways of
      // expressing the same idea). The conversion is
      // performed by the Set accessors.
      // ---------------------------------------------
      public void SetNbRepaintsPerSecond (long NbTimesPerSecond)
      {
         if (NbTimesPerSecond != 0L)
            SetRepaintInterval (1000L / NbTimesPerSecond);
         else
            SetRepaintInterval (DEFAULT_REPAINT_INTERVAL);
      }
      public void SetRepaintInterval (long interval)
      {
         RepaintInterval = interval;
      }

      // ---------------------------------------------
      // Accessors (Get)
      // ---------------------------------------------

      public boolean GetStopping ()
      {
         return Stopping;
      }

      // ---------------------------------------------
      // See remarks on the repaint interval (above)
      // ---------------------------------------------
      public long GetNbRepaintsPerSecond ()
      {
         return 1000L / GetRepaintInterval ();
      }
      public long GetRepaintInterval ()
      {
         return RepaintInterval;
      }

      // ---------------------------------------------
      // Default constructor
      // ---------------------------------------------
      public PainterThread ()
      {
         super ();
         SetRepaintInterval (DEFAULT_REPAINT_INTERVAL);
         Stopping = false;
      }

      // ---------------------------------------------
      // The thread itself -- method run().
      // ---------------------------------------------
      public void run ()
      {
         while (!GetStopping ())
         {
            BalloonWorld.repaint ();
            try
            {
               sleep (GetRepaintInterval ());
            }
            catch (Exception e)
            {
            }
         }
         SetStopping (false);
      }

      // ---------------------------------------------
      // Method ShutDown() wil be called by the owner
      // of this thread to inform it that it's time to
      // stop. This method only concludes when the
      // thread has stopped properly
      // ---------------------------------------------
      public void Shutdown ()
      {
         SetStopping (true);
         while (Stopping)
         {
            yield ();
         }
      }

      // ---------------------------------------------
      // Events (slider). Updates the refresh rate
      // according to the slider's value
      // ---------------------------------------------
      public void stateChanged (ChangeEvent e)
      {
         SetNbRepaintsPerSecond (((JSlider) e.getSource ()).getValue ());
      }
   }




   // ------------------------------------------------
   // ------------------------------------------------
   // Each instance of class BalloonThread will do the
   // job of making a balloon grow, and then move
   // around in its balloonpanel until the end of the
   // program (or at least until the thread stops).
   // ------------------------------------------------
   // ------------------------------------------------


   // ------------------------------------------------
   // Each entry in array Balloons[] represents a
   // balloon's execution thread.
   // ------------------------------------------------
   private BalloonThread [] Balloons;

   private class BalloonThread extends Thread implements ChangeListener
   {
      // ---------------------------------------------
      // Class constants
      // ---------------------------------------------
      private static final int
         MAX_RADIUS     = 150 // arbitrary (we could even let the user decide)
,        STATE_STOPPED  = 0
,        STATE_GROWING  = 1
,        STATE_MOVING   = 2
,        STATE_STOPPING = 3
,        BALLOON_INITIAL_RADIUS = 10 // arbitrary
,        BALLOON_GROWTH_RATE = 20 // times per second; arbitrary
,        BALLOON_MOVE_RATE   = 20 // times per second; arbitrary
,        SPEED_DEFAULT  = 1 // pixel per movement; arbitrary
;

      // ---------------------------------------------
      // Instance attributes
      // ---------------------------------------------
      private int Radius, CenterX, CenterY;
      private int State;

      // Balloon direction (+1, -1)
      private int DirectionX, DirectionY;

      // Balloon speed (same in x and in y. We could make it
      // more interesting :)
      private int Speed;

      // These will be true if the balloon is colliding with
      // the borders of the panel, false otherwise. I'm not
      // checking for collisions with other balloons (to keep
      // things simple)
      private boolean IsCollidingX, IsCollidingY;

      // ---------------------------------------------
      // Accessors (Get)
      // ---------------------------------------------
      public int GetRadius ()
      {
         return Radius;
      }
      public int GetCenterX ()
      {
         return CenterX;
      }
      public int GetCenterY ()
      {
         return CenterY;
      }
      private int GetState ()
      {
         return State;
      }
      public int GetSpeed ()
      {
         return Speed;
      }
      public boolean GetIsCollidingX ()
      {
         return IsCollidingX;
      }
      public boolean GetIsCollidingY ()
      {
         return IsCollidingY;
      }
      public int GetDirectionX ()
      {
         return DirectionX;
      }
      public int GetDirectionY ()
      {
         return DirectionY;
      }
      // ---------------------------------------------
      // Accessors (Set)
      // ---------------------------------------------
      public void SetRadius (int r)
      {
         Radius = (r < MAX_RADIUS)? r: MAX_RADIUS;
      }
      public void SetCenterX (int x)
      {
         CenterX = x;
      }
      public void SetCenterY (int y)
      {
         CenterY = y;
      }
      private void SetState (int n)
      {
         State = n;
      }
      private void SetSpeed (int sp)
      {
         Speed = sp;
      }
      public void SetIsCollidingX (boolean boom)
      {
         IsCollidingX = boom;
      }
      public void SetIsCollidingY (boolean boom)
      {
         IsCollidingY = boom;
      }
      private void SetDirectionX (int dx)
      {
         DirectionX = dx;
      }
      private void SetDirectionY (int dy)
      {
         DirectionY = dy;
      }

      // ---------------------------------------------
      // Other utility methods
      // ---------------------------------------------

      // ---------------------------------------------
      // GetIsColliding () returns true if the balloon
      // is colliding on either axis, false otherwise.
      // ---------------------------------------------
      public boolean GetIsColliding ()
      {
         return GetIsCollidingX () || GetIsCollidingY ();
      }

      // ---------------------------------------------
      // StopThisBalloon () tells itself (thread) to
      // stop, then waits until this stopping operation
      // has been completed
      // ---------------------------------------------
      public void StopThisBalloon ()
      {
         SetState (STATE_STOPPING);
         while (GetState () != STATE_STOPPED)
         {
            yield ();
         }
      }

      // ---------------------------------------------
      // UpdateDirection () changes the direction of
      // the balloon (either axis) if it collides with
      // the sides of the panel
      // ---------------------------------------------
      private void UpdateDirection ()
      {
         if (GetIsCollidingX ())
         {
            SetDirectionX (- GetDirectionX ());
            SetIsCollidingX (false);
         }
         if (GetIsCollidingY ())
         {
            SetDirectionY (- GetDirectionY ());
            SetIsCollidingY (false);
         }
      }

      // ---------------------------------------------
      // LaunchThisBallon () sets the balloon in motion
      // ---------------------------------------------
      public void LaunchThisBallon ()
      {
         SetDirectionX (r.nextInt (2) != 0? 1 : -1);
         SetDirectionY (r.nextInt (2) != 0? 1 : -1);
         SetState (STATE_MOVING);
      }

      // ---------------------------------------------
      // Default constructor
      // ---------------------------------------------
      public BalloonThread (int x, int y)
      {
         super ();
         SetCenterX (x);
         SetCenterY (y);
         SetRadius (BALLOON_INITIAL_RADIUS);
         SetState (STATE_GROWING);
         SetIsCollidingX (false);
         SetIsCollidingY (false);
         SetSpeed (SPEED_DEFAULT);
      }

      // ---------------------------------------------
      // The thread itself -- method run(). Note that
      // this method sleeps at every iteration of its
      // central loop, but the sleep time can vary
      // according to the events (the value of the
      // balloon's slider; whether the balloon is
      // growing or moving; etc.), so giving a default
      // value to variable SleepTime is important.
      // ---------------------------------------------
      public void run ()
      {
         final int DEFAULT_SLEEP_TIME = 1; // milliseconds
         boolean Stop = false;

         while (!Stop)
         {
            int SleepTime = DEFAULT_SLEEP_TIME;
            switch (GetState ())
            {
               case STATE_GROWING:
                  if (GetRadius () < MAX_RADIUS)
                  {
                     SetRadius (GetRadius () + 1);
                  }
                  SleepTime = 1000 / BALLOON_GROWTH_RATE;
                  break;
               case STATE_MOVING:
                  if (GetIsColliding ())
                  {
                     UpdateDirection ();
                  }
                  SetCenterX (GetCenterX () + GetDirectionX () * Speed);
                  SetCenterY (GetCenterY () + GetDirectionY () * Speed);
                  SleepTime = 1000 / BALLOON_MOVE_RATE;
                  break;
               case STATE_STOPPING:
                  Stop = true;
                  break;
               default: // bad, bad error
                  SetState (STATE_STOPPING);
            }
            try
            {
               sleep (SleepTime);
            }
            catch (Exception e)
            {
            }
         }
         SetState (STATE_STOPPED);
      }

      // ---------------------------------------------
      // Events (slider). Updates the balloon's speed
      // according to the slider's value
      // ---------------------------------------------
      public void stateChanged (ChangeEvent e)
      {
         SetSpeed ( ((JSlider) e.getSource ()).getValue () );
      }
   }
   // ------------------------------------------------
   // ------------------------------------------------



   // ------------------------------------------------
   // GUI set-up methods
   // ------------------------------------------------

   // ------------------------------------------------
   // CreateControlPanel () creates the panel that will
   // hold the SliderPanel instances
   // ------------------------------------------------
   private JPanel CreateControlPanel ()
   {
      // One slider bunch per potential balloon, plus one for the
      // refresh rate control
      final int CONTROL_PANEL_NB_COLUMNS = MAX_BALLOONS + 1,
                CONTROL_PANEL_NB_ROWS    = 1;

      final String REFRESH_SLIDER_TITLE = "Refreshes per second";

      JPanel ControlPanel = new JPanel ();
      ControlPanel.setLayout (new GridLayout (CONTROL_PANEL_NB_COLUMNS,
                                              CONTROL_PANEL_NB_ROWS));

      // Refresh rate slider
      RefreshSlider = new SliderPanel (REFRESH_SLIDER_TITLE, 1, 100, 10);
      RefreshSlider.GetSlider ().setEnabled (true);
      ControlPanel.add (RefreshSlider);

      // Balloon sliders
      Sliders = new SliderPanel [MAX_BALLOONS];
      for (int i = 0; i < Sliders.length; i++)
      {
         Sliders[i] = new SliderPanel  ("Ballon #"+ i +" speed", 1, 100, 50);
         Sliders[i].GetSlider ().setEnabled (false);
         ControlPanel.add (Sliders[i]);
      }

      return ControlPanel;
   }

   // ------------------------------------------------
   // init() sets-up the GUI
   // ------------------------------------------------
   public void init ()
   {
      // Set-up of the status label
      StatusLabel = new JLabel ("");
      SetStatus (STATUS_DEFAULT);
      getContentPane ().add (StatusLabel, BorderLayout.SOUTH);

      // Set-up of the control panel
      getContentPane ().add (CreateControlPanel (), BorderLayout.EAST);

      // Set-up of the balloon panel
      BalloonWorld = new BalloonPanel ();
      getContentPane ().add (BalloonWorld, BorderLayout.CENTER);

      // Set-up of event handlers
      BalloonWorld.addMouseListener (
         new MouseAdapter()
         {
            public void mousePressed (MouseEvent e)
            {
               if (IsFullOfBalloons ())
               {
                  SetStatus (STATUS_FULL);
               }
               else
               {
                  StartGrowingABalloon (e.getX (), e.getY ());
               }
            }
            public void mouseReleased (MouseEvent e)
            {
               if (BalloonReady ())
               {
                  LaunchBalloon ();
               }
            }
         }
      );
      addWindowListener (
         new WindowAdapter ()
         {
            public void windowClosing (WindowEvent e)
            {
               StopAllBalloons ();
               ThePainter.Shutdown ();
               System.exit (0);
            }
         }
      );
   }

   

   // ------------------------------------------------
   // Balloon-using attributes and methods :)
   // ------------------------------------------------

   // ABallonIsGrowing is true if a balloon is growing,
   // false otherwise. No more than one balloon can
   // grow at any moment since we must use the mouse
   // to make a balloon grow (whew!)
   private boolean ABallonIsGrowing;

   private void SetBalloonGrowing (boolean grows)
   {
      ABallonIsGrowing = grows;
   }
   private boolean GetBalloonGrowing ()
   {
      return ABallonIsGrowing;
   }

   // A balloon is ready if one is growing
   private boolean BalloonReady ()
   {
      return GetBalloonGrowing ();
   }

   // StopAllBalloons () loops through the Balloons[]
   // threads and stops each of them. Note that we use
   // GetNbBalloons () for the number of balloons in
   // execution, since the Balloons[] array might not
   // be filled yet
   private void StopAllBalloons ()
   {
      for (int i = 0; i < GetNbBalloons (); i++)
      {
         Balloons[i].StopThisBalloon (); // blocks until stopped
      }
   }

   // StartGrowingABalloon (x,y) starts growing a balloon
   // at center (x, y). This creates the thread for the
   // the balloon, makes it listen for the slider that will
   // controls its speed (but the slider is not enabled yet;
   // it will be enabled once the balloon is launched), and
   // starts the balloon's thread.
   private void StartGrowingABalloon (int x, int y)
   {
      Balloons[GetNbBalloons ()] = new BalloonThread (x, y);
      Sliders[GetNbBalloons ()].GetSlider ().addChangeListener (Balloons[GetNbBalloons ()]);
      Sliders[GetNbBalloons ()].GetSlider ().setValue (Balloons[GetNbBalloons ()].GetSpeed ());
      Balloons[GetNbBalloons ()].start ();
      SetNbBalloons (GetNbBalloons () + 1);
      SetStatus (STATUS_DEFAULT);
      SetBalloonGrowing (true);
   }

   // LaunchBalloon () laucnhes the balloon that is currently
   // growing. It also makes the slider available for the user
   // to be able to change the balloon's speed
   private void LaunchBalloon ()
   {
      Balloons[GetNbBalloons () - 1].LaunchThisBallon ();
      Sliders[GetNbBalloons () - 1].GetSlider ().setEnabled (true);
      SetBalloonGrowing (false);
   }

   // ------------------------------------------------
   // Default constructor
   // ------------------------------------------------
   public ThreadDemo ()
   {
      this (DEFAULT_TITLE);
   }

   // ------------------------------------------------
   // Cosntructor
   // ------------------------------------------------
   public ThreadDemo (String Title)
   {
      super (Title);
      init ();
      SetNbBalloons (0); // initially: no balloons

      // We create the BalloonThread array, but not
      // the BalloonThread instances themselves
      Balloons = new BalloonThread[MAX_BALLOONS];

      // ThePainter is created and started right away.
      // the RefreshSlider's valeu is adjusted initially
      // to match the default refresh rate of ThePainter
      ThePainter = new PainterThread ();
      RefreshSlider.GetSlider ().addChangeListener (ThePainter);
      RefreshSlider.GetSlider ().setValue ((int) ThePainter.GetNbRepaintsPerSecond ());
      ThePainter.start ();

      // Initially, no balloon is growing
      SetBalloonGrowing (false);
   }

   // ------------------------------------------------
   // Main entry point of the program (straightforward)
   // ------------------------------------------------
   public static void main (String [] args)
   {
      final int WINDOW_WIDTH = 600
,               WINDOW_HEIGHT = 500
;
      ThreadDemo td = new ThreadDemo ();
      td.setSize (WINDOW_WIDTH, WINDOW_HEIGHT);
      td.setVisible (true);
   }
}

